@@ -22,10 +22,21 @@ def reqdiff(version_tag_from, version_tag_to):
     _reqs.print_reqdiff(*rdiff)
 
 
-def reqcheck(version, spec=False, strict=False, python_version='3.6'):
+def reqcheck(version, python_version='3.6', override=None):
     m = re.search(r'^[\d]\.[\d]$', python_version)
     if not m:
         raise exception.WrongPythonVersion()
+
+    override_pkgs = None
+    if override:
+        try:
+            with open(override, 'r') as override_file:
+                override_pkgs = yaml.load(override_file,
+                                          Loader=yaml.SafeLoader)
+        except Exception as e:
+            e.strerror = 'Unable to load the override file (%s)' % e.strerror
+            raise
+
     if version.upper() == 'XXX':
         if 'upstream' in git.remotes():
             current_branch = git.current_branch()
@@ -33,7 +44,9 @@ def reqcheck(version, spec=False, strict=False, python_version='3.6'):
             if branch != 'master':
                 branch = 'stable/{}'.format(branch)
             version = 'upstream/{}'.format(branch)
-            check = _reqs.reqcheck_spec(python_version, ref=version)
+            check = _reqs.reqcheck_spec(python_version,
+                                        ref=version,
+                                        override_pkgs=override_pkgs)
         else:
             m = re.search(r'/([^/]+)_distro', os.getcwd())
             if not m:
@@ -41,19 +54,29 @@ def reqcheck(version, spec=False, strict=False, python_version='3.6'):
                                           why="failed to parse current path")
             path = '../%s/requirements.txt' % m.group(1)
             log.info("Delorean detected. Using %s" % path)
-            check = _reqs.reqcheck_spec(python_version, reqs_txt=path)
+            check = _reqs.reqcheck_spec(python_version,
+                                        reqs_txt=path,
+                                        ref=version,
+                                        override_pkgs=override_pkgs)
     else:
-        check = _reqs.reqcheck_spec(python_version, ref=version)
-    format = None
+        check = _reqs.reqcheck_spec(python_version,
+                                    ref=version,
+                                    override_pkgs=override_pkgs)
+    return {'check': check}
+
+
+def reqcheck_print(check=None, output=None, spec=False, strict=False):
+    if output not in ['spec', 'json', 'text']:
+        raise exception.WrongOutputFormat()
     if spec:
-        format = 'spec'
-    _reqs.print_reqcheck(*check, format=format)
+        output = 'spec'
+    _reqs.print_reqcheck(*check, format=output)
     if strict:
         # missing
-        if len(check[-1]) > 0:
+        if len(check[3]) > 0:
             raise exception.ReqCheckMissingDependencies()
         # mismatch
-        if len(check[-2]) > 0:
+        if len(check[2]) > 0:
             raise exception.ReqCheckMismatchingDependencies()
 
 
