@@ -206,6 +206,7 @@ class Spec(object):
         self._fn = fn
         self._txt = txt
         self._rpmspec = None
+        self._contains_subpkg = None
 
     @property
     def fn(self):
@@ -288,7 +289,7 @@ class Spec(object):
 
     def _create_new_magic_comment(self, name, value):
         # check to see if we have any magic comments in right slot
-        # after SourceX and before Patch Y - if so insert at begining block
+        # after SourceX and before Patch Y - if so insert at beginning block
         # otherwise insert a new block as before
 
         if re.findall(self.RE_IN_MAGIC_COMMENTS, self._txt, flags=re.M):
@@ -640,6 +641,10 @@ class Spec(object):
         vr = self.get_vr(epoch=epoch)
         return '%s-%s' % (name, vr)
 
+    def get_name(self):
+        """get Name from .spec"""
+        return self.get_tag('Name', expand_macros=True)
+
     def new_changelog_entry(self, user, email, changes=[]):
         changes_str = "\n".join(map(lambda x: "- %s" % x, changes)) + "\n"
         date = time.strftime('%a %b %d %Y')
@@ -744,3 +749,205 @@ class Spec(object):
             except KeyError:
                 pass
         return requires
+
+    def edit_python_requires_version_by_name(self, name, version=''):
+        name = name.split('-', 1)[1]
+        repl = r'\1 {}\3' if version else r'\1\3'
+        self._txt, n = re.subn(
+            r'^(%s:\s+python.*-%s)\s*([<>=!]*\s[,.\d\w]*)?(\n)'
+            % (re.escape('Requires'), name),
+            repl.format(version),
+            self.txt,
+            flags=re.M)
+        return n > 0
+
+    def remove_python_requires_by_name(self, name):
+        name = name.split('-', 1)[1]
+        repl = r''
+        self._txt, n = re.subn(r'^%s:\s+python.*-%s(\s+[<>=!]*\s[,.\d\w]*)?\n'
+                               % (re.escape('Requires'), name),
+                               repl,
+                               self.txt,
+                               flags=re.M)
+        if n:
+            return n > 0
+        return False
+
+    def get_subpackages(self):
+        """
+        Retrieve all the subpackages present in the .spec file.
+        Return a dictionary which contains subpackage name as key, and
+        the beginning and ending indexes of the subpkg in the .spec file
+        as value.
+        """
+        beginning_of_subpkg, end_of_subpkg, subpackages = '', '', {}
+        txt_list = self.txt.split('\n')
+        main_package_name = self.get_name()
+
+        all_subpkgs = re.findall(r'^%package.*$', self.txt, re.M)
+        if not all_subpkgs:
+            return None
+
+        for subpkg in all_subpkgs:
+            beginning_of_subpkg = txt_list.index(subpkg)
+            for line in txt_list[beginning_of_subpkg:]:
+                if re.match('%description', line):
+                    end_of_subpkg = txt_list.index(line, beginning_of_subpkg)
+                    break
+
+            # If there is no '-n' option to the %package directive, we prepend
+            # the main package name to the subpackage one.
+            m = re.search(r'^%package\s+(-n\s+)?(.*)', subpkg)
+            if not m.group(1):
+                subpkg = '{}-{}'.format(main_package_name, m.group(2))
+            else:
+                subpkg = m.group(2)
+            subpackages[subpkg] = (beginning_of_subpkg, end_of_subpkg)
+        return subpackages
+
+    def guess_main_python_subpackage(self, main_py_subpkg=None):
+        # To guess which is the main python subpackage, we can only rely on RPM
+        # guidelines and mechanisms.
+        # Below the ordered search criteria:
+        # 1. Subpackage name starting with python and having the less dashes
+        #    (most cases e.g: python3-foo, python3-foo-doc, python3-foo-tests)
+        # 2. If 1. returns nothing, we take the first subpackage found (most
+        #    likely to be the main one).
+        # 3. Finally, we ensure it does not require another subpackage (meaning
+        #    it's not the main one). If it's not, we iterate recursively with
+        #    the subpackage required, until finding the first one which does
+        #    not require a subpackage.
+        python_subpackages, counter = [], 0
+
+        if not self._contains_subpkg:
+            self._contains_subpkg = self.get_subpackages()
+
+        if main_py_subpkg is None:
+            try:
+                python_subpackages = [x for x in self._contains_subpkg if
+                                      x.startswith('python')]
+            except TypeError:
+                return None
+
+            for _py_subpkg in python_subpackages:
+                if counter == 0:
+                    counter = _py_subpkg.count('-')
+                    main_py_subpkg = _py_subpkg
+                elif _py_subpkg.count('-') < counter:
+                    counter = _py_subpkg.count('-')
+                    main_py_subpkg = _py_subpkg
+
+        try:
+            start_index, end_index = self._contains_subpkg[main_py_subpkg]
+        except KeyError:
+            main_py_subpkg = list(self._contains_subpkg.keys())[0]
+            start_index, end_index = self._contains_subpkg[main_py_subpkg]
+
+        txt_list = self.txt.split('\n')
+        for line in txt_list[start_index:end_index + 1]:
+            if not re.search(r'^Requires:\s+(.*)\s+=\s+(.*)', line):
+                continue
+            line = re.sub(r'%{name}', self.get_name(), line)
+            m = re.search(r'^Requires:\s+(.*)\s+=\s+(.*)', line)
+            if m:
+                for subpkg_name in self._contains_subpkg.keys():
+                    if subpkg_name == m.group(1):
+                        return self.guess_main_python_subpackage(subpkg_name)
+        return main_py_subpkg
+
+    def find_last_dependency(self, dep_type, starting_index=None,
+                             ending_index=None):
+        """
+        Find last dependency (Requires, BuildRequires, Suggests, etc) within
+        the .spec file. We can search in a specific range by specifying the
+        starting and ending indexes.
+        Dependencies which are within a conditional block are ignored.
+        Return the index position of the last found dependency, else None.
+        """
+        last_line_index, excluded, nested_if_statement = None, False, 0
+        txt_list = self.txt.split('\n')
+        try:
+            txt_range = txt_list[starting_index:ending_index + 1]
+        except TypeError:
+            txt_range = txt_list
+
+        for index, line in enumerate(txt_range):
+            if line.startswith('{}:'.format(dep_type)) and not excluded:
+                last_line_index = index
+            elif line.startswith('%if'):
+                excluded = True
+                nested_if_statement += 1
+            elif line.startswith('%endif'):
+                nested_if_statement -= 1
+                excluded = False if nested_if_statement == 0 else True
+
+        try:
+            return starting_index + last_line_index
+        except TypeError:
+            return last_line_index
+
+    def insert_dependency_after(self, dep, line_position, dep_type='Requires',
+                                py_version='3.6'):
+        """
+        Take the dependency type (Requires, BuildRequires, Suggests, etc),
+        the dependency name and the line position.
+        It insert the dependency after the line position in the .spec file, and
+        returns True if successful, else False.
+        """
+        txt_list = self.txt.split('\n')
+        try:
+            last_dep = txt_list[line_position]
+        except IndexError:
+            return False
+        major_py_version = py_version.split('.')[0]
+        dep = re.sub(r'^python-(.*)$',
+                     r'python{}-\g<1>'.format(major_py_version),
+                     dep)
+        # For the sake of consistency, we get the number of spaces (\s*)
+        # between the last dependency type (ending with ":") and its associated
+        # value.
+        m = re.search(r'^(.*):(\s*)(.*)', last_dep)
+        nbr_of_spaces = m.group(2) if m else ' '
+        txt_list.insert(line_position + 1, '{}:{}{}'.format(dep_type,
+                                                            nbr_of_spaces,
+                                                            dep))
+        self._txt = '\n'.join(txt_list)
+        return True
+
+    def add_python_requires(self, requires, subpkg_name=None):
+        """
+        Add Requires after the last found Requires found in the main package.
+        If a python subpackage is provided as argument, the method will add the
+        Requires into the subpackage-specific section.
+        If no Requires found, the method will add it after the last found BR.
+        If no BR found, it will add it after the BuildArch tag.
+        The method returns True if the Requires has been added, else False.
+        """
+        if not self._contains_subpkg:
+            self._contains_subpkg = self.get_subpackages()
+
+        try:
+            starting_index, ending_index = self._contains_subpkg[
+                subpkg_name]
+        except KeyError:
+            # The provided subpackage is not found, we limit the spec file
+            # from its beginning to the first subpackage found. That way, we
+            # ignore subpackages when looking for the last found Requires (or
+            # BuildRequires), and inserting the new one.
+            starting_index = 0
+            first_subpkg = list(self._contains_subpkg.keys())[0]
+            ending_index = self._contains_subpkg[first_subpkg][0]
+        except TypeError:
+            # There is no subpackages, the working area is the whole .spec file
+            starting_index, ending_index = '', ''
+
+        # Add new Requires after last Requires, or BR or BuildArch.
+        for dep_type in ['Requires', 'BuildRequires', 'BuildArch']:
+            last_dep = self.find_last_dependency(dep_type,
+                                                 starting_index,
+                                                 ending_index)
+            if last_dep:
+                return self.insert_dependency_after(requires,
+                                                    last_dep,
+                                                    'Requires')
+        raise exception.CouldNotAddPythonRequires()
