@@ -620,3 +620,304 @@ def test_get_requires_not_provided_02(tmpdir):
         for name, version in packages:
             expected[name] = version
         assert got == expected
+
+
+def test_remove_python_requires_by_name(tmpdir):
+    txt = '\n'.join(['Requires:     python-sqlalchemy',
+                     'Requires:     python-prettytable >= 1.0.10',
+                     'Requires:     python-iso8601'])
+    spec = specfile.Spec(txt=txt)
+    got = spec.remove_python_requires_by_name('python-sqlalchemy')
+    assert got is True
+    got = spec.remove_python_requires_by_name('python-prettytable')
+    assert got is True
+    assert 'Requires:     python-sqlalchemy' not in spec.txt
+    assert 'Requires:     python-prettytable >= 1.0.10' not in spec.txt
+    assert 'Requires:     python-iso8601' in spec.txt
+
+
+def test_remove_python_requires_by_name_false(tmpdir):
+    txt = '\n'.join(['Requires:     python-sqlalchemy >= 1.0.10',
+                     'Requires:     python-prettytable',
+                     'Requires:     python-iso8601'])
+    spec = specfile.Spec(txt=txt)
+    got = spec.remove_python_requires_by_name('python-argparse')
+    assert got is False
+
+
+def test_edit_python_requires_version_by_name_true(tmpdir):
+    txt = '\n'.join(['Requires:     python-sqlalchemy >= 1.0.10',
+                     '',
+                     'BuildRequires:     python-sqlalchemy',
+                     'Requires:     python-prettytable',
+                     'Requires:     python3-iso8601 >= 1.0.0',
+                     'Requires:     python-osc-lib >= 1.0.0'])
+    spec = specfile.Spec(txt=txt)
+    got = spec.edit_python_requires_version_by_name('python-sqlalchemy',
+                                                    '>= 1.0.12')
+    assert got is True
+    assert 'Requires:     python-sqlalchemy >= 1.0.12\n\n' in spec.txt
+    assert 'BuildRequires:     python-sqlalchemy\n' in spec.txt
+    got = spec.edit_python_requires_version_by_name('python-prettytable',
+                                                    '>= 1.0.1')
+    assert got is True
+    assert 'Requires:     python-prettytable >= 1.0.1\n' in spec.txt
+    got = spec.edit_python_requires_version_by_name('python3-iso8601', '')
+    assert got is True
+    assert 'Requires:     python3-iso8601\n' in spec.txt
+
+
+def test_edit_python_requires_version_by_name_false(tmpdir):
+    txt = '\n'.join(['Requires:     python-sqlalchemy >= 1.0.10',
+                     'Requires:     python-prettytable',
+                     'Requires:     python-iso8601',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.edit_python_requires_version_by_name('python-argparse')
+    assert got is False
+
+
+def test_get_subpackages_1(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     '%package -n        python3-foo',
+                     'Summary:           foo summary',
+                     'Requires:          python-bar1',
+                     '%description -n    python-foo',
+                     '%{common_desc}',
+                     '',
+                     '%package           doc',
+                     'Requires:          python-bar2',
+                     '%description       doc',
+                     '%{common_desc}',
+                     '',
+                     '%package test',
+                     'Requires:          python-bar3',
+                     '%description       test',
+                     '%{common_desc}'])
+    spec = specfile.Spec(txt=txt)
+    subpkgs = spec.get_subpackages()
+    assert subpkgs['python3-foo'] == (1, 4)
+    assert subpkgs['openstack-foo-doc'] == (7, 9)
+    assert subpkgs['openstack-foo-test'] == (12, 14)
+
+
+def test_get_subpackages_2(tmpdir):
+    txt = '\n'.join(['Name:            openstack-foo',
+                     'Requires:        python3-foo1',
+                     'BuildRequires:   python-baz1'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.get_subpackages() is None
+
+
+def test_find_last_dependency_1(tmpdir):
+    txt = '\n'.join(['Requires:      python-foo1',
+                     'Requires:      python-foo2',
+                     'BuildRequires: python-bar1',
+                     'PreReq:        is-deprecated',
+                     'Requires:      python-foo3',
+                     'Requires:      python-foo3'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.find_last_dependency('Requires') == 5
+
+
+def test_find_last_dependency_2(tmpdir):
+    txt = '\n'.join(['BuildRequires: python-setuptools',
+                     'PreReq:        is-deprecated'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.find_last_dependency('Requires') is None
+
+
+def test_find_last_dependency_3(tmpdir):
+    txt = '\n'.join(['Requires:      python-foo1',
+                     'Requires:      python-foo2',
+                     'PreReq:        is-deprecated',
+                     'Requires:      python-bar1',
+                     'BuildRequires: python-baz1'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.find_last_dependency('PreReq', 0, 2) == 2
+
+
+def test_find_last_dependency_4(tmpdir):
+    txt = '\n'.join(['Requires:      python-foo1',
+                     'Requires:      python-foo2',
+                     '%if 0%{?fedora}',
+                     'Requires: qemu-kvm-core >= 2.8.0',
+                     '%if 0%{?with_doc}',
+                     'Requires:      python-foo3',
+                     '%endif',
+                     'Requires:      python-foo4',
+                     '%endif'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.find_last_dependency('Requires', 0, 8) == 1
+
+
+def test_insert_dependency_after_1(tmpdir):
+    txt = '\n'.join(['Requires:     python3-foo1',
+                     'Requires:     python3-foo2',
+                     'PreReq:       is-deprecated',
+                     'Requires:     python3-bar1',
+                     'BuildRequires:   python-baz1'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.insert_dependency_after('python-bar2', 3) is True
+    assert ("Requires:     python3-bar1\n"
+            "Requires:     python3-bar2\n") in spec.txt
+
+
+def test_insert_dependency_after_2(tmpdir):
+    txt = '\n'.join(['BuildArch:    noarch',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    assert spec.insert_dependency_after('python-bar2', 1) is True
+    assert ("BuildArch:    noarch\n"
+            "\n"
+            "Requires: python3-bar2") in spec.txt
+
+
+def test_insert_dependency_after_3(tmpdir):
+    txt = '\n'.join(['BuildArch:    noarch'])
+    spec = specfile.Spec(txt=txt)
+    assert spec.insert_dependency_after('python-bar2', 1) is False
+
+
+def test_add_python_requires_1_in_subpkg_after_last_found_requires(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     '%package -n        python3-foo',
+                     'Summary:           foo summary',
+                     'BuildRequires:     python3-bar1',
+                     'Requires:          python3-bar2',
+                     '%description -n    python3-foo',
+                     '%{common_desc}',
+                     '',
+                     '%package -n        python3-foo-tests',
+                     'Summary:           foo summary',
+                     'BuildRequires:     python3-bar3',
+                     'Requires:          python3-bar4',
+                     '%description -n    python3-foo-tests',
+                     '%{common_desc}'])
+    spec = specfile.Spec(txt=txt)
+    got = spec.add_python_requires('python-argparse', 'python3-foo')
+    assert got is True
+    assert ("Requires:          python3-bar2\n"
+            "Requires:          python3-argparse\n"
+            "%description -n    python3-foo") in spec.txt
+
+
+def test_add_python_requires_2_in_main_package_with_subpkgs_present(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     '%package -n        python3-foo',
+                     'BuildRequires:     python3-bar1',
+                     'Requires:          python3-bar2',
+                     '%description -n    python3-foo',
+                     '%{common_desc}',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.add_python_requires('python-argparse')
+    assert got is True
+    assert ("BuildArch:         noarch\n"
+            "Requires:         python3-argparse") in spec.txt
+
+
+def test_add_python_requires_3_in_main_package_without_subpkgs_present(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     'BuildRequires:     python3-bar1',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.add_python_requires('python-argparse', '')
+    assert got is True
+    assert ("BuildRequires:     python3-bar1\n"
+            "Requires:     python3-argparse") in spec.txt
+
+
+def test_add_python_requires_4_raising_could_not_add_message(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'Summary:       This is foo summary.'])
+    with pytest.raises(exception.CouldNotAddPythonRequires):
+        spec = specfile.Spec(txt=txt)
+        got = spec.add_python_requires('python-argparse', '')
+
+
+def test_guess_main_python_subpackage_1(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     '%package -n        python3-foo',
+                     'Requires:          python3-bar1',
+                     '%description -n    python3-foo',
+                     '%{common_desc}',
+                     '%package -n        python3-foo-test',
+                     'Requires:          python3-bar2',
+                     '%description -n    python3-foo-test',
+                     '%{common_desc}',
+                     '%package -n        python3-foo-doc',
+                     'Requires:          python3-bar3',
+                     '%description -n    python3-foo-doc',
+                     '%{common_desc}',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.guess_main_python_subpackage()
+    assert got == 'python3-foo'
+
+
+def test_guess_main_python_subpackage_2_no_subpkgs_starting_w_python(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     '%package -n        openstack-foo',
+                     'Requires:          python3-bar1',
+                     '%description -n    openstack-foo',
+                     '%{common_desc}',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.guess_main_python_subpackage()
+    assert got == "openstack-foo"
+
+
+def test_guess_main_python_subpackage_3_no_subpackages(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.guess_main_python_subpackage()
+    assert got is None
+
+
+def test_guess_main_python_subpackage_4_do_not_start_with_python(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     '%package           common',
+                     'Requires:          python3-bar1',
+                     '%description       common',
+                     '%{common_desc}',
+                     '%package -n        python3-foo-test',
+                     'Requires: %{name}-common = %{version}-%{release}',
+                     '%description -n    python3-foo-test',
+                     '%{common_desc}',
+                     '%package -n        python3-foo-doc',
+                     'Requires: %{name}-common = %{version}-%{release}',
+                     '%description -n    python3-foo-doc',
+                     '%{common_desc}',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.guess_main_python_subpackage()
+    assert got == 'openstack-foo-common'
+
+
+def test_guess_main_python_subpackage_5_at_the_end(tmpdir):
+    txt = '\n'.join(['Name:              openstack-foo',
+                     'BuildArch:         noarch',
+                     '%package -n        openstack-foo-test',
+                     'Requires: %{name}-common = %{version}-%{release}',
+                     '%description -n    openstack-foo-test',
+                     '%{common_desc}',
+                     '%package -n        openstack-foo-doc',
+                     'Requires: %{name}-common = %{version}-%{release}',
+                     '%description -n    openstack-foo-doc',
+                     '%{common_desc}',
+                     '%package           common',
+                     'Requires:          python3-bar1',
+                     '%description       common',
+                     '%{common_desc}',
+                     ''])
+    spec = specfile.Spec(txt=txt)
+    got = spec.guess_main_python_subpackage()
+    assert got == 'openstack-foo-common'
